﻿using Minio;
using Minio.AspNetCore;
using Minio.Exceptions;
using PaydayBackend.Utils;

namespace PaydayBackend.Services;

public interface IStorageService
{
    public Task<bool> UploadFile(MemoryStream fileStream, string bucket, string fileName);
    public Task<MemoryStream?> GetFile(string bucket, string fileName);
    public Task<bool> RemoveFile(string bucket, string fileName);
    public Task<bool> IsFileExist(string bucket, string fileName);
}

public class StorageService : IStorageService
{
    private MinioClient _minio;

    public StorageService(IMinioClientFactory minioClientFactory)
    {
        _minio = minioClientFactory.CreateClient();
    }

    public async Task<MemoryStream?> GetFile(string bucket, string fileName)
    {
        if (!await IsFileExist(bucket, fileName))
            return null;

        try
        {
            var outStream = new MemoryStream();
            GetObjectArgs getObjectArgs = new GetObjectArgs()
                .WithBucket(bucket)
                .WithObject(fileName)
                .WithCallbackStream((stream) =>
                {
                    stream.CopyTo(outStream);
                    outStream.Position = 0;
                    stream.Close();
                });
            await _minio.GetObjectAsync(getObjectArgs);

            return outStream;
        }
        catch (MinioException e)
        {
            Console.WriteLine("File upload error: {0}", e.Message);
            return null;
        }
    }

    public async Task<bool> RemoveFile(string bucket, string fileName)
    {
        try
        {
            RemoveObjectArgs rmArgs = new RemoveObjectArgs()
                .WithBucket(bucket)
                .WithObject(fileName);
            await _minio.RemoveObjectAsync(rmArgs);
            Console.WriteLine($"File \"{fileName}\" removed successfully");
            return true;
        }
        catch (MinioException e)
        {
            Console.WriteLine($"File \"{fileName}\" remove error: " + e);
            return false;
        }
    }

    public async Task<bool> IsFileExist(string bucket, string fileName)
    {
        try
        {
            // Проверка, существует ли объект
            // Если объект не найден, statObjectArgs генерирует исключение
            StatObjectArgs statObjectArgs = new StatObjectArgs()
                .WithBucket(bucket)
                .WithObject(fileName);
            await _minio.StatObjectAsync(statObjectArgs);
            return true;
        }
        catch (MinioException e)
        {
            Console.WriteLine($"File \"{fileName}\" not exist: " + e);
            return false;
        }
    }

    public async Task<bool> UploadFile(MemoryStream fileStream, string bucket, string fileName)
    {
        fileStream.Position = 0;

        try
        {
            // Создаём ведро если его нет
            var beArgs = new BucketExistsArgs().WithBucket(bucket);
            bool found = await _minio.BucketExistsAsync(beArgs).ConfigureAwait(false);
            if (!found)
            {
                var mbArgs = new MakeBucketArgs().WithBucket(bucket);
                await _minio.MakeBucketAsync(mbArgs).ConfigureAwait(false);
            }

            // Загружаем файл
            var putObjectArgs = new PutObjectArgs()
                .WithBucket(bucket)
                .WithObject(fileName)
                .WithStreamData(fileStream)
                .WithObjectSize(fileStream.Length)
                .WithContentType(FileHelper.GetMimeType(fileName));
            await _minio.PutObjectAsync(putObjectArgs).ConfigureAwait(false);
            Console.WriteLine("File uploaded successfully: " + fileName);
            fileStream.Close();
            return true;
        }
        catch (MinioException e)
        {
            Console.WriteLine("File upload error: {0}", e.Message);
            fileStream.Close();
            return false;
        }
    }
}